package meerkat;

import meerkat.bulletinboard.BulletinBoardClient;
import meerkat.protobuf.BulletinBoardAPI;
import util.RegistryTags;

import java.text.ParseException;
import java.util.List;
import java.util.Map;

import static util.CollectionMessagesUtils.*;

/**
 * Created by Vladimir Eliezer Tokarev  on 2/19/2016.
 * Object that handles the GetGroups or the GetPersonIDDetails of the simpleRegistry
 */
public class RelevantDataCallBack implements BulletinBoardClient.ClientCallback<List<BulletinBoardAPI.BulletinBoardMessage>> {
    public VoterRegistry.RegistryCallBack callback;

    /**
     * Init BooleanHandler
     * @param callback  voter registry callback object
     */
    public RelevantDataCallBack(VoterRegistry.RegistryCallBack callback) {
        this.callback = callback;
    }

    /**
     * Checks if the given list have tags of type GROUP_ID_TAG or else
     * @param msg List<BulletinBoardAPI.BulletinBoardMessage>
     * @return  true if the messages are with GROUP_ID_TAG tags
     */
    private boolean isAddToGroupsList(List<BulletinBoardAPI.BulletinBoardMessage> msg) {
        for (String tag : msg.get(0).getMsg().getTagList()) {
            if(tag.contains(RegistryTags.GROUP_ID_TAG.toString()))
            {
                return true;
            }
        }
        return false;
    }

    /**
     *  Checks if the list of messages is of type GROUP_ID_TAG if it is then calls
     *          HandleResult with mapping of the latest groups, else calls to HandleResult with
     *          the latest tag from this list (in case of personal data)
     *
     * @param msg List<BulletinBoardAPI.BulletinBoardMessage>
     */
    @Override
    public void handleCallback(List<BulletinBoardAPI.BulletinBoardMessage> msg) {
        List<VoterRegistryMessage> messages = ConvertToVoterRegistryMessages(msg);

        if(isAddToGroupsList(msg)){
            try {
                Map<String, VoterRegistryMessage> map = GetLatestGroupsActions(messages);
                List<String> groupsOfUser = GetListOfGroupIds(map);
                callback.HandleResult(groupsOfUser);
            } catch (ParseException e) {
                callback.HandleResult(null);
            }
        }
        try {
            callback.HandleResult(GetLatestMessage(messages));
        } catch (ParseException e) {
            e.printStackTrace();
        }
    }

    /**
     * Calls the callback HandleResult method with false because the post method failed
     * @param t the exception data that have been thrown during the failure of the post method
     */
    @Override
    public void handleFailure(Throwable t) {
        callback.HandleResult(null);
    }
}
