package meerkat;

/**
 * Created by Vladimir Eliezer Tokarev on 1/22/2016.
 * provides voters management options
 */
public interface VoterRegistry {

    /**
     * Created by Vladimir Eliezer Tokarev on 1/22/2016.
     * This interface will handle the end of methods of RegistryInstance
     */
    interface RegistryCallBack<T> {
        boolean ActionSucceed = false;
        void HandleResult(T result);
    }

    /**
     * Adds new voter to the bulletin-board
     *      Passes true to callBack.HandleResult  if the actions succeeded else false
     *
     * @param voterInfo    protobuff object that represents voter information
     * @param callBack     when the adding voter done callBack.HandleResult will be called
     * @return void
     */
    void AddVoter(RegistryMessages.VoterInfo voterInfo, RegistryCallBack callBack);

    /**
     * Adding given voter to given group
     *        Passes the group  to callBack.HandleResult  if the actions succeeded else null
     *
     * @param voterGroup protobuff object that is coupling of voterId to groupId
     * @param callBack when the adding voter done callBack.HandleResult will be called
     * @return true if the adding action succeeded else return false
     */
    void AddToGroup(RegistryMessages.VoterGroup voterGroup, RegistryCallBack callBack);

    /**
     * Sets that the voter have voted
     *          Passes true to callBack.HandleResult  if the actions succeeded else false
     *
     * @param voterId       id tag string
     * @param callBack when the adding voter done callBack.HandleResult will be called
     * @return true if the set voted succeed else false
     */
    void SetVoted(RegistryMessages.VoterID voterId, RegistryCallBack callBack);

    /**
     * Requests all the groups that the given id voter is in
     *         Passes wanted groups to callback.HandleResult if the actions succeeded else null
     *
     * @param groupID       id tag string
     * @param callBack when the adding voter done callBack.HandleResult will be called
     */
    void GetGroups(RegistryMessages.GroupID groupID, RegistryCallBack callBack);

    /**
     * Retrieves list of strings that represents voter
     *          Passes wanted data to callback.HandleResult if the actions succeeded else null
     *
     * @param voterID       id tag string
     * @param callBack when the adding voter done callBack.HandleResult will be called
     * @return list of strings (empty list if the lookup failed)
     */
    void GetPersonIDDetails(RegistryMessages.VoterID voterID, RegistryCallBack callBack);
}

