package meerkat;

import meerkat.protobuf.BulletinBoardAPI;
import util.AccurateTimestamp;
import util.RegistryTags;

import java.sql.Timestamp;
import java.text.ParseException;

/**
 * Created by Vladimir Eliezer Tokarev on 1/15.2016
 * this class wraps BasicMessage and gives the ability to find wanted tags
 */
public class VoterRegistryMessage {

    public BulletinBoardAPI.UnsignedBulletinBoardMessage base;

    public VoterRegistryMessage(BulletinBoardAPI.BulletinBoardMessage message) {
        BulletinBoardAPI.UnsignedBulletinBoardMessage.Builder unsignedBase =
                BulletinBoardAPI.UnsignedBulletinBoardMessage.newBuilder().addAllTag(message.getMsg().getTagList());
        unsignedBase.setData(message.getMsg().getData());
        base = unsignedBase.build();
    }

    /**
     * Gets the wanted tag from given basic message
     *
     * @param tagName the name of the tag
     * @return string
     */
    public String GetWantedTagFromBasicMessage(String tagName) {
        for (String tag : base.getTagList()) {
            if (tag.contains(tagName)) {
                return tag;
            }
        }
        return null;
    }

    /**
     * Gets the timestamp of the tag adding
     *
     * @return Timestamp
     * @throws ParseException
     */
    public Timestamp GetBasicMessageActionTimestamp() throws ParseException {
        for (String tag : base.getTagList()) {
            if (tag.contains(RegistryTags.ACTION_TIMESTAMP_TAG.toString())) {
                String[] tagParts = tag.split(" ");

                String timestamp = tagParts[tagParts.length - 2] + " " + tagParts[tagParts.length - 1];
                return AccurateTimestamp.GetTimestampFromString(timestamp);
            }
        }
        return null;

    }

    /**
     * Checks if the given message have the ADD_TO_GROUP_TAG
     *
     * @return true when ADD_TO_GROUP_TAG exist else false
     */
    public boolean IsGroupAdding() {
        for (String tag : base.getTagList()) {
            if (tag.contains(RegistryTags.ADD_TO_GROUP_TAG.toString())) {
                return true;
            }
        }
        return false;
    }
}
