package util;

import meerkat.VoterRegistryMessage;
import meerkat.protobuf.BulletinBoardAPI;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by Vladimir Eliezer Tokarev  on 1/15/2016.
 * adds extra functionality to Messages collections
 */
public abstract class CollectionMessagesUtils {

    /**
     * Converts lost of UnsignedBulletinBoardMessage to VoterRegistryMessages
     * @param messages list<VoterRegistryMessage>
     * @return List<VoterRegistryMessage>
     */
   public static List<VoterRegistryMessage> ConvertToVoterRegistryMessages(
           List<BulletinBoardAPI.BulletinBoardMessage> messages){
       return messages.stream().map(VoterRegistryMessage::new).collect(Collectors.toList());
   }

    /**
     * Gets map of GroupId to basicMessage, where the basicMessages are the last actions for those groups
     * @param messages List<VoterRegistryMessage>
     * @return     Map{String:VoterRegistryMessage}
     * @throws ParseException
     */
    public static Map<String, VoterRegistryMessage> GetLatestGroupsActions(List<VoterRegistryMessage> messages) throws ParseException {
        Map<String, VoterRegistryMessage> groupIdToMessage = new HashMap<>();

        // iterate trough all the messages and put into the map the last updated groups actions
        for (VoterRegistryMessage message : messages) {
            String groupId = message.GetWantedTagFromBasicMessage(RegistryTags.GROUP_ID_TAG.toString());
            VoterRegistryMessage temp = groupIdToMessage.get(groupId);

            if (temp != null && temp != message) {
                if (temp.GetBasicMessageActionTimestamp().before(message.GetBasicMessageActionTimestamp())) {
                    groupIdToMessage.put(groupId, message);
                }
            }
            groupIdToMessage.put(groupId, message);
        }
        return groupIdToMessage;
    }

    /**
     * Gets list of groups ids of the basicMessages that carried the adding to group tag
     * @param groupIdToMessage Map<String, VoterRegistryMessage>
     * @return List<String>
     */
     public static List<String> GetListOfGroupIds(Map<String, VoterRegistryMessage>  groupIdToMessage) {
         List<String> groupsIds = new ArrayList<>();

         groupIdToMessage.values().stream().filter(VoterRegistryMessage::IsGroupAdding).forEach(message -> {
             String groupId = message.GetWantedTagFromBasicMessage(RegistryTags.GROUP_ID_TAG.toString());
             groupsIds.add(groupId);
         });
         return groupsIds;
     }

    public static VoterRegistryMessage GetLatestMessage(List<VoterRegistryMessage> messages) throws ParseException {
        if (messages.size() == 0 ){
            return null;
        }
        VoterRegistryMessage LatestMessage = messages.get(0);

        for (VoterRegistryMessage message : messages) {
            if (message.GetBasicMessageActionTimestamp().before(LatestMessage.GetBasicMessageActionTimestamp())) {
                LatestMessage = message;
            }
        }
        return LatestMessage;
    }

}
